@@ -48,6 +48,12 @@ module Milter
       end
     end
 
+    def on_event_loop_created
+      signal_connect("event-loop-created") do |_client|
+        yield(_client)
+      end
+    end
+
     def reload
       reload_callbacks.each do |callback|
         callback.call(self)
@@ -235,6 +241,14 @@ module Milter
     def remove_event(tag)
       @context.event_loop.remove(tag)
     end
+
+    def [](name)
+      @context[name]
+    end
+
+    def worker_id
+      @context.client.worker_id
+    end
   end
 
   class ClientSessionContext
