@@ -108,6 +108,7 @@ module Milter
           @run_gc_on_maintain = true
           @handle_signal = true
           @maintained_hooks = []
+          @event_loop_created_hooks = []
         end
 
         def setup(client)
@@ -115,7 +116,7 @@ module Milter
           client.connection_spec = @connection_spec
           client.unix_socket_group = @unix_socket_group
           client.unix_socket_mode = @unix_socket_mode if @unix_socket_mode
-          client.event_loop_backend = @event_loop_backend
+          client.event_loop_backend = resolved_event_loop_backend
           client.default_packet_buffer_size = @packet_buffer_size
           client.pid_file = @pid_file
           client.maintenance_interval = @maintenance_interval
@@ -132,6 +133,14 @@ module Milter
               end
             end
           end
+          unless @event_loop_created_hooks.empty?
+            client.on_event_loop_created do |_client, loop|
+              @event_loop = loop
+              @event_loop_created_hooks.each do |hook|
+                hook.call(loop)
+              end
+            end
+          end
           if @max_file_descriptors.nonzero?
             begin
               Process.setrlimit(Process::RLIMIT_NOFILE, @max_file_descriptors)
@@ -142,6 +151,14 @@ module Milter
           end
         end
 
+        # For Ruby 1.8.5 + old Ruby/GLib2. It should be
+        # removed when CentOS 5 support is dropped.
+        def resolved_event_loop_backend
+          Milter::ClientEventLoopBackend.values.find do |value|
+            value.nick == @event_loop_backend.to_s
+          end
+        end
+
         def update_location(key, reset, deep_level)
           @base_configuration.update_location(key, reset, deep_level + 1)
         end
@@ -372,6 +389,15 @@ module Milter
         end
       end
 
+      class << self
+        def guard(fallback_value=nil)
+          yield
+        rescue Exception => error
+          Milter::Logger.error(error)
+          fallback_value
+        end
+      end
+
       attr_reader :milter, :security, :log, :database
       def initialize(configuration)
         @configuration = configuration
@@ -398,13 +424,6 @@ module Milter
                              "<#{path}>")
       end
 
-      def guard(fallback_value=nil)
-        yield
-      rescue Exception => error
-        Milter::Logger.error(error)
-        fallback_value
-      end
-
       def environment
         @configuration.environment
       end
@@ -481,13 +500,17 @@ module Milter
           @configuration.remove_unix_socket_on_close = remove
         end
 
+        def daemon?
+          @configuration.daemon?
+        end
+
         def daemon=(boolean)
           update_location("daemon", false)
           @configuration.daemon = boolean
         end
 
-        def daemon?
-          @configuration.daemon?
+        def pid_file
+          @configuration.pid_file
         end
 
         def pid_file=(pid_file)
@@ -495,8 +518,8 @@ module Milter
           @configuration.pid_file = pid_file
         end
 
-        def pid_file
-          @configuration.pid_file
+        def maintenance_interval
+          @configuration.maintenance_interval
         end
 
         def maintenance_interval=(n_sessions)
@@ -505,8 +528,8 @@ module Milter
           @configuration.maintenance_interval = n_sessions
         end
 
-        def maintenance_interval
-          @configuration.maintenance_interval
+        def suspend_time_on_unacceptable
+          @configuration.suspend_time_on_unacceptable
         end
 
         def suspend_time_on_unacceptable=(seconds)
@@ -515,8 +538,8 @@ module Milter
           @configuration.suspend_time_on_unacceptable = seconds
         end
 
-        def suspend_time_on_unacceptable
-          @configuration.suspend_time_on_unacceptable
+        def max_connections
+          @configuration.max_connections
         end
 
         def max_connections=(n_connections)
@@ -525,8 +548,8 @@ module Milter
           @configuration.max_connections = n_connections
         end
 
-        def max_connections
-          @configuration.max_connections
+        def max_file_descriptors
+          @configuration.max_file_descriptors
         end
 
         def max_file_descriptors=(n_descriptors)
@@ -535,8 +558,8 @@ module Milter
           @configuration.max_file_descriptors = n_descriptors
         end
 
-        def max_file_descriptors
-          @configuration.max_file_descriptors
+        def connection_check_interval
+          @configuration.connection_check_interval
         end
 
         def connection_check_interval=(interval)
@@ -583,6 +606,15 @@ module Milter
           @configuration.maintained_hooks << guarded_hook
         end
 
+        def event_loop_created(hook=Proc.new)
+          guarded_hook = Proc.new do |configuration, loop|
+            ConfigurationLoader.guard do
+              hook.call(loop)
+            end
+          end
+          @configuration.event_loop_created_hooks << guarded_hook
+        end
+
         private
         def update_location(key, reset, deep_level=2)
           full_key = "milter.#{key}"
