@@ -248,6 +248,13 @@ security.effective_user = nil
 # default
 security.effective_group = nil
 
+# default
+log.level = "none"
+# default
+log.use_syslog = true
+# default
+log.syslog_facility = nil
+
 # default
 manager.connection_spec = #{@configuration.manager_connection_spec.inspect}
 # default
@@ -295,6 +302,21 @@ controller.unix_socket_group = nil
 controller.remove_unix_socket_on_create = true
 # default
 controller.remove_unix_socket_on_close = true
+
+# default
+database.type = nil
+# default
+database.name = nil
+# default
+database.host = nil
+# default
+database.port = nil
+# default
+database.path = nil
+# default
+database.user = nil
+# default
+database.password = nil
 EOD
                  @configuration.dump)
   end
@@ -316,6 +338,11 @@ EOD
     controller_unix_socket_group = __LINE__ + 1
     loader.controller.unix_socket_group = "nogroup"
 
+    database_type = __LINE__ + 1
+    loader.database.type = "sqlite3"
+    database_name = __LINE__ + 1
+    loader.database.name = ":memory:"
+
     s25r_lines = {}
     s25r_lines[:define] = __LINE__ + 1
     loader.define_applicable_condition("S25R") do |condition|
@@ -360,6 +387,13 @@ security.effective_user = "nobody"
 # #{__FILE__}:#{effective_group_line}
 security.effective_group = "nogroup"
 
+# default
+log.level = "none"
+# default
+log.use_syslog = true
+# default
+log.syslog_facility = nil
+
 # default
 manager.connection_spec = #{@configuration.manager_connection_spec.inspect}
 # default
@@ -408,6 +442,21 @@ controller.remove_unix_socket_on_create = true
 # default
 controller.remove_unix_socket_on_close = true
 
+# #{__FILE__}:#{database_type}
+database.type = "sqlite3"
+# #{__FILE__}:#{database_name}
+database.name = ":memory:"
+# default
+database.host = nil
+# default
+database.port = nil
+# default
+database.path = nil
+# default
+database.user = nil
+# default
+database.password = nil
+
 # #{__FILE__}:#{s25r_lines[:define]}
 define_applicable_condition("S25R") do |condition|
   # #{__FILE__}:#{s25r_lines[:description]}
