@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -70,7 +70,7 @@ class TestRedHatInitDetector < Test::Unit::TestCase
                                      "--connection-spec", "",
                                      "--user-name", "",
                                      "--group-name", "",
-                                     "--socket-group-name", "",
+                                     "--unix-socket-group", "",
                                     ].join(" ") + " ",
                    "OPTION_ARGS" => "",
                    "RETVAL" => "$?",
@@ -130,7 +130,7 @@ EOC
                                      "--connection-spec", "",
                                      "--user-name", "",
                                      "--group-name", "",
-                                     "--socket-group-name", "",
+                                     "--unix-socket-group", "",
                                     ].join(" ") + " ",
                    "OPTION_ARGS" => "",
                    "RETVAL" => "$?",
