@@ -85,8 +85,9 @@ G_BEGIN_DECLS
  */
 typedef enum
 {
+    MILTER_CLIENT_EVENT_LOOP_BACKEND_DEFAULT,
     MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
-    MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV,
+    MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV
 } MilterClientEventLoopBackend;
 
 /**
@@ -114,6 +115,7 @@ typedef enum
     MILTER_CLIENT_ERROR_DAEMONIZE,
     MILTER_CLIENT_ERROR_NOT_LISTENED_YET,
     MILTER_CLIENT_ERROR_PROCESS,
+    MILTER_CLIENT_ERROR_PID_FILE
 } MilterClientError;
 
 typedef struct _MilterClientClass    MilterClientClass;
@@ -140,7 +142,10 @@ struct _MilterClientClass
     void   (*listen_started)              (MilterClient *client,
                                            struct sockaddr *address,
                                            socklen_t        address_size);
-    gchar *(*get_default_connection_spec) (MilterClient *client);
+    const gchar *(*get_connection_spec)   (MilterClient *client);
+    gboolean (*set_connection_spec)       (MilterClient *client,
+                                           const gchar  *spec,
+                                           GError      **error);
     guint  (*get_unix_socket_mode)        (MilterClient *client);
     void   (*set_unix_socket_mode)        (MilterClient *client,
                                            guint         mode);
@@ -169,6 +174,8 @@ struct _MilterClientClass
     void   (*maintain)                    (MilterClient *client);
     void   (*sessions_finished)           (MilterClient *client,
                                            guint         n_finished_sessions);
+    void   (*event_loop_created)          (MilterClient *client,
+                                           MilterEventLoop *loop);
     MilterClientEventLoopBackend
            (*get_event_loop_backend)      (MilterClient *client);
     void   (*set_event_loop_backend)      (MilterClient *client,
@@ -182,6 +189,15 @@ struct _MilterClientClass
                                           (MilterClient *client,
                                            guint         size);
     GPid   (*fork)                        (MilterClient *client);
+    const gchar *(*get_pid_file)          (MilterClient *client);
+    void   (*set_pid_file)                (MilterClient *client,
+                                           const gchar  *pid_file);
+    gboolean (*is_remove_pid_file_on_exit)(MilterClient *client);
+    void   (*set_remove_pid_file_on_exit) (MilterClient *client,
+                                           gboolean      remove);
+    gboolean (*is_run_as_daemon)          (MilterClient *client);
+    void   (*set_run_as_daemon)           (MilterClient *client,
+                                           gboolean      daemon);
 };
 
 GQuark               milter_client_error_quark       (void);
@@ -658,26 +674,11 @@ gboolean             milter_client_run               (MilterClient  *client,
  * Returns: %TRUE if main loop is quitted successfully,
  * %FALSE otherwise.
  *
- * Depracated: 1.6.2: Use milter_client_run() instead.
+ * Deprecated: 1.6.2: Use milter_client_run() instead.
  */
 gboolean             milter_client_main              (MilterClient  *client);
 #endif
 
-/**
- * milter_client_run_master:
- * @client: a %MilterClient.
- * @error: return location for an error, or %NULL.
- *
- * Runs main loop in parent process. If the main loop isn't
- * quitted successfully and @error is not %NULL, error
- * detail is stored into @error.
- *
- * Returns: %TRUE if main loop is quitted successfully,
- * %FALSE otherwise.
- */
-gboolean             milter_client_run_master        (MilterClient  *client,
-                                                      GError       **error);
-
 /**
  * milter_client_shutdown:
  * @client: a %MilterClient.
@@ -764,26 +765,83 @@ guint                milter_client_get_n_processing_sessions
 gboolean             milter_client_is_processing     (MilterClient  *client);
 
 /**
- * milter_client_start_syslog:
+ * milter_client_get_syslog_facility:
  * @client: a %MilterClient.
- * @identify: the identify name in syslog.
- * @facility: the syslog facility. Maybe %NULL.
  *
- * Starts logging by syslog with @identify name.
+ * Gets the syslog facility name of the @client.
+ *
+ * Returns: syslog facility name.
  */
-void                 milter_client_start_syslog      (MilterClient  *client,
-                                                      const gchar   *identify,
+const gchar          *milter_client_get_syslog_facility
+                                                     (MilterClient  *client);
+
+/**
+ * milter_client_set_syslog_identify:
+ * @client: a %MilterClient.
+ * @identify: the identify in syslog.
+ *
+ * Sets the syslog identify of the @client.
+ */
+void                  milter_client_set_syslog_identify
+                                                     (MilterClient  *client,
+                                                      const gchar   *identify);
+
+/**
+ * milter_client_get_syslog_identify:
+ * @client: a %MilterClient.
+ *
+ * Gets the syslog identify name of the @client.
+ *
+ * Returns: syslog identify name.
+ */
+const gchar          *milter_client_get_syslog_identify
+                                                     (MilterClient  *client);
+
+/**
+ * milter_client_set_syslog_facility:
+ * @client: a %MilterClient.
+ *
+ * Sets the syslog facility name of the @client.
+ */
+void                  milter_client_set_syslog_facility
+                                                     (MilterClient  *client,
                                                       const gchar   *facility);
 
 /**
- * milter_client_get_process_loop:
+ * milter_client_start_syslog:
+ * @client: a %MilterClient.
+ *
+ * Starts logging by syslog with @identify name.
+ */
+void                 milter_client_start_syslog      (MilterClient  *client);
+
+/**
+ * milter_client_stop_syslog:
+ * @client: a %MilterClient.
+ *
+ * Stops logging by syslog.
+ */
+void                 milter_client_stop_syslog       (MilterClient  *client);
+
+/**
+ * milter_client_get_syslog_enabled:
+ * @client: a %MilterClient.
+ *
+ * Gets whether syslog is enabled.
+ *
+ * Returns: %TRUE if syslog is enabled, %FALSE otherwise.
+ */
+gboolean             milter_client_get_syslog_enabled(MilterClient  *client);
+
+/**
+ * milter_client_get_event_loop:
  * @client: a %MilterClient.
  *
  * Gets the %MilterEventLoop for processing requests.
  *
  * Returns: the %MilterEventLoop for processing requests.
  */
-MilterEventLoop     *milter_client_get_process_loop  (MilterClient  *client);
+MilterEventLoop     *milter_client_get_event_loop   (MilterClient  *client);
 
 /**
  * milter_client_get_event_loop_backend:
@@ -819,7 +877,7 @@ void                 milter_client_set_event_loop_backend
 guint                milter_client_get_n_workers     (MilterClient  *client);
 
 /**
- * milter_client_get_n_workers:
+ * milter_client_set_n_workers:
  * @client: a %MilterClient.
  * @n_workers: the number of worker processes.
  *
@@ -889,6 +947,67 @@ MilterClientCustomForkFunc
  */
 guint                milter_client_get_worker_id        (MilterClient *client);
 
+/**
+ * milter_client_get_pid_file:
+ * @client: a %MilterClient.
+ *
+ * Gets the PID file of @client.
+ *
+ * Returns: the PID file of @client.
+ */
+const gchar         *milter_client_get_pid_file         (MilterClient  *client);
+
+/**
+ * milter_client_set_pid_file:
+ * @client: a %MilterClient.
+ * @pid_file: the PID file of @client.
+ *
+ * Sets the PID file of @client.
+ */
+void                 milter_client_set_pid_file         (MilterClient  *client,
+                                                         const gchar   *pid_file);
+
+/**
+ * milter_client_is_remove_pid_file_on_exit:
+ * @client: a %MilterClient.
+ *
+ * Gets whether removing PID file after main loop is quitted.
+ *
+ * Returns: %TRUE if removing PID file on exit, %FALSE otherwise.
+ */
+gboolean             milter_client_is_remove_pid_file_on_exit
+                                                     (MilterClient  *client);
+
+/**
+ * milter_client_set_remove_pid_file_on_exit:
+ * @client: a %MilterClient.
+ * @remove: %TRUE if removing PID file on exit.
+ *
+ * Sets whether removing PID file after main loop is quitted.
+ */
+void                 milter_client_set_remove_pid_file_on_exit
+                                                     (MilterClient  *client,
+                                                      gboolean       remove);
+
+/**
+ * milter_client_is_run_as_daemon:
+ * @client: a %MilterClient.
+ *
+ * Gets whether running as a daemon or not.
+ *
+ * Returns: %TRUE if running as a daemon, %FALSE otherwise.
+ */
+gboolean             milter_client_is_run_as_daemon  (MilterClient  *client);
+
+/**
+ * milter_client_set_run_as_daemon:
+ * @client: a %MilterClient.
+ * @daemon: %TRUE if runnning as a daemon.
+ *
+ * Sets whether running as a daemon.
+ */
+void                 milter_client_set_run_as_daemon (MilterClient  *client,
+                                                      gboolean       daemon);
 
 G_END_DECLS
 
