@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -147,7 +147,7 @@ milter_headers_copy (MilterHeaders *headers)
     for (node = priv->header_list; node; node = g_list_next(node)) {
         MilterHeader *header = node->data;
 
-        milter_headers_add_header(copied_headers, header->name, header->value);
+        milter_headers_append_header(copied_headers, header->name, header->value);
     }
 
     return copied_headers;
@@ -184,7 +184,7 @@ milter_headers_find (MilterHeaders *headers,
                                     (GCompareFunc)milter_header_compare);
     if (!found_node)
         return NULL;
-    return (MilterHeader*)found_node->data;
+    return (MilterHeader *)(found_node->data);
 }
 
 static gint
@@ -273,14 +273,43 @@ milter_headers_remove (MilterHeaders *headers,
 }
 
 gboolean
-milter_headers_add_header(MilterHeaders *headers,
-                          const gchar *name,
-                          const gchar *value)
+milter_headers_add_header (MilterHeaders *headers,
+                           const gchar *name,
+                           const gchar *value)
+{
+    MilterHeadersPrivate *priv;
+    GList *node, *same_name_header = NULL;
+
+    priv = MILTER_HEADERS_GET_PRIVATE(headers);
+
+    for (node = priv->header_list; node; node = g_list_next(node)) {
+        MilterHeader *header = node->data;
+        if (g_strcasecmp(header->name, name) == 0) {
+            same_name_header = node;
+            break;
+        }
+    }
+
+    if (same_name_header) {
+        priv->header_list = g_list_insert_before(priv->header_list,
+                                                 same_name_header,
+                                                 milter_header_new(name, value));
+    } else {
+        priv->header_list = g_list_append(priv->header_list,
+                                          milter_header_new(name, value));
+    }
+
+    return TRUE;
+}
+
+gboolean
+milter_headers_append_header (MilterHeaders *headers,
+                              const gchar *name,
+                              const gchar *value)
 {
     MilterHeadersPrivate *priv;
 
     priv = MILTER_HEADERS_GET_PRIVATE(headers);
-
     priv->header_list = g_list_append(priv->header_list,
                                       milter_header_new(name, value));
 
