@@ -1013,7 +1013,7 @@ do_header (MilterManagerTestScenario *scenario, const gchar *group)
     milter_server_context_header(MILTER_SERVER_CONTEXT(server), name, value);
     milter_manager_leader_header(leader, name, value);
 
-    milter_manager_test_server_add_header(server, name, value);
+    milter_manager_test_server_append_header(server, name, value);
 
     assert_response(scenario, group);
     assert_alive(scenario, group);
@@ -1026,7 +1026,8 @@ do_header (MilterManagerTestScenario *scenario, const gchar *group)
         expected_headers, actual_headers,
         milter_manager_test_pair_equal,
         (GCutInspectFunction)milter_manager_test_pair_inspect,
-        NULL);
+        NULL,
+        cut_message("[%s] <%s>:<%s>", group, name, value));
 }
 
 static void
@@ -1483,6 +1484,8 @@ data_scenario_end_of_message_action (void)
                  g_strdup("delete-header.txt"), g_free,
                  "delete-header - evaluation",
                  g_strdup("delete-header-evaluation.txt"), g_free,
+                 "delete-header - many",
+                 g_strdup("delete-header-many.txt"), g_free,
                  NULL);
 
     cut_add_data("change-from",
