@@ -28,6 +28,10 @@
 #include <glib/gi18n.h>
 #include <fcntl.h>
 
+#ifdef HAVE_LOCALE_H
+#  include <locale.h>
+#endif
+
 #include <glib/gprintf.h>
 
 #include <sys/socket.h>
@@ -47,10 +51,17 @@ static gint n_threads = 0;
 static gchar *connect_host = NULL;
 static struct sockaddr *connect_address = NULL;
 static socklen_t connect_address_length = 0;
+static GHashTable *connect_macros = NULL;
 static gchar *helo_host = NULL;
+static GHashTable *helo_macros = NULL;
 static gchar *envelope_from = NULL;
+static GHashTable *envelope_from_macros = NULL;
 static gchar **recipients = NULL;
+static GHashTable *envelope_recipient_macros = NULL;
+static GHashTable *data_macros = NULL;
+static GHashTable *end_of_header_macros = NULL;
 static gchar **body_chunks = NULL;
+static GHashTable *end_of_message_macros = NULL;
 static gchar *unknown_command = NULL;
 static gchar *authenticated_name = NULL;
 static gchar *authenticated_type = NULL;
@@ -162,67 +173,44 @@ set_macro (gpointer key, gpointer value, gpointer user_data)
 static void
 set_macros_for_connect (MilterProtocolAgent *agent)
 {
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_CONNECT,
-                                     "{daemon_name}", g_get_prgname(),
-                                     "{if_name}", "localhost",
-                                     "{if_addr}", "127.0.0.1",
-                                     "j", "milter-test-server",
-                                     NULL);
+    milter_protocol_agent_set_macros_hash_table(agent, MILTER_COMMAND_CONNECT,
+                                                connect_macros);
 }
 
 static void
 set_macros_for_helo (MilterProtocolAgent *agent)
 {
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_HELO,
-                                     "{tls_version}", "0",
-                                     "{cipher}", "0",
-                                     "{cipher_bits}", "0",
-                                     "{cert_subject}", "cert_subject",
-                                     "{cert_issuer}", "cert_issuer",
-                                     NULL);
+    milter_protocol_agent_set_macros_hash_table(agent, MILTER_COMMAND_HELO,
+                                                helo_macros);
 }
 
 static void
 set_macros_for_envelope_from (MilterProtocolAgent *agent)
 {
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
-                                     "i", "i",
-                                     "{mail_mailer}", "mail_mailer",
-                                     "{mail_host}", "mail_host",
-                                     "{mail_addr}", "mail_addr",
-                                     NULL);
-
-    if (authenticated_name)
-        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
-                                         "{auth_authen}", authenticated_name,
-                                         NULL);
-    if (authenticated_type)
-        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
-                                         "{auth_type}", authenticated_type,
-                                         NULL);
-    if (authenticated_author)
-        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
-                                         "{auth_author}", authenticated_author,
-                                         NULL);
+    milter_protocol_agent_set_macros_hash_table(agent,
+                                                MILTER_COMMAND_ENVELOPE_FROM,
+                                                envelope_from_macros);
 }
 
 static void
-set_macros_for_envelope_recipient (MilterProtocolAgent *agent)
+set_macros_for_envelope_recipient (MilterProtocolAgent *agent,
+                                   const gchar *recipient)
 {
-    gchar *recipients_address;
+    milter_protocol_agent_set_macros_hash_table(agent,
+                                                MILTER_COMMAND_ENVELOPE_RECIPIENT,
+                                                envelope_recipient_macros);
 
-    recipients_address = g_strjoinv(",", recipients);
     milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_RECIPIENT,
-                                     "{rcpt_mailer}", "rcpt_mailer",
-                                     "{rcpt_host}", "rcpt_host",
-                                     "{rcpt_addr}", recipients_address,
+                                     "{rcpt_addr}", recipient,
                                      NULL);
-    g_free(recipients_address);
 }
 
 static void
 set_macros_for_data (MilterProtocolAgent *agent)
 {
+    milter_protocol_agent_set_macros_hash_table(agent,
+                                                MILTER_COMMAND_DATA,
+                                                data_macros);
 }
 
 static void
@@ -233,6 +221,9 @@ set_macros_for_header (MilterProtocolAgent *agent)
 static void
 set_macros_for_end_of_header (MilterProtocolAgent *agent)
 {
+    milter_protocol_agent_set_macros_hash_table(agent,
+                                                MILTER_COMMAND_END_OF_HEADER,
+                                                end_of_header_macros);
 }
 
 static void
@@ -243,9 +234,9 @@ set_macros_for_body (MilterProtocolAgent *agent)
 static void
 set_macros_for_end_of_message (MilterProtocolAgent *agent)
 {
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_END_OF_MESSAGE,
-                                     "{msg-id}", "msg-id",
-                                     NULL);
+    milter_protocol_agent_set_macros_hash_table(agent,
+                                                MILTER_COMMAND_END_OF_MESSAGE,
+                                                end_of_message_macros);
 }
 
 static void
@@ -317,14 +308,15 @@ send_envelope_recipient (MilterServerContext *context, ProcessData *data)
 {
     gchar *recipient;
 
-    set_macros_for_envelope_recipient(MILTER_PROTOCOL_AGENT(context));
-    if (milter_option_get_step(data->option) & MILTER_STEP_NO_ENVELOPE_RECIPIENT)
-        return FALSE;
-
     recipient = recipients[data->current_recipient];
     if (!recipient)
         return FALSE;
 
+    set_macros_for_envelope_recipient(MILTER_PROTOCOL_AGENT(context),
+                                      recipient);
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_ENVELOPE_RECIPIENT)
+        return FALSE;
+
     milter_server_context_envelope_recipient(context, recipient);
     data->current_recipient++;
     return TRUE;
@@ -1021,6 +1013,79 @@ parse_connect_address_arg (const gchar *option_name,
     return success;
 }
 
+static gboolean
+parse_macro_arg (GHashTable *macros,
+                 const gchar *option_name,
+                 const gchar *value,
+                 gpointer data,
+                 GError **error)
+{
+    gboolean success = TRUE;
+    gchar **macro;
+
+    macro = g_strsplit(value, ":", 2);
+    if (!macro[1]) {
+        g_set_error(error,
+                    G_OPTION_ERROR,
+                    G_OPTION_ERROR_BAD_VALUE,
+                    "macro name and value delimiter ':' doesn't exist: <%s>",
+                    value);
+        success = FALSE;
+    } else {
+        g_hash_table_insert(macros, g_strdup(macro[0]), g_strdup(macro[1]));
+    }
+    g_strfreev(macro);
+
+    return success;
+}
+
+static gboolean
+parse_connect_macro_arg (const gchar *option_name,
+                         const gchar *value,
+                         gpointer data,
+                         GError **error)
+{
+    return parse_macro_arg(connect_macros, option_name, value, data, error);
+}
+
+static gboolean
+parse_helo_macro_arg (const gchar *option_name,
+                      const gchar *value,
+                      gpointer data,
+                      GError **error)
+{
+    return parse_macro_arg(helo_macros, option_name, value, data, error);
+}
+
+static gboolean
+parse_envelope_from_macro_arg (const gchar *option_name,
+                               const gchar *value,
+                               gpointer data,
+                               GError **error)
+{
+    return parse_macro_arg(envelope_from_macros, option_name, value,
+                           data, error);
+}
+
+static gboolean
+parse_envelope_recipient_macro_arg (const gchar *option_name,
+                                    const gchar *value,
+                                    gpointer data,
+                                    GError **error)
+{
+    return parse_macro_arg(envelope_recipient_macros, option_name, value,
+                           data, error);
+}
+
+static gboolean
+parse_data_macro_arg (const gchar *option_name,
+                      const gchar *value,
+                      gpointer data,
+                      GError **error)
+{
+    return parse_macro_arg(data_macros, option_name, value, data, error);
+}
+
 static gboolean
 parse_header_arg (const gchar *option_name,
                   const gchar *value,
@@ -1046,6 +1111,26 @@ parse_header_arg (const gchar *option_name,
     return TRUE;
 }
 
+static gboolean
+parse_end_of_header_macro_arg (const gchar *option_name,
+                               const gchar *value,
+                               gpointer data,
+                               GError **error)
+{
+    return parse_macro_arg(end_of_header_macros, option_name, value,
+                           data, error);
+}
+
+static gboolean
+parse_end_of_message_macro_arg (const gchar *option_name,
+                                const gchar *value,
+                                gpointer data,
+                                GError **error)
+{
+    return parse_macro_arg(end_of_message_macros, option_name, value,
+                           data, error);
+}
+
 static gboolean
 set_envelope_from (const gchar *from)
 {
@@ -1169,22 +1254,29 @@ parse_mail_contents (const gchar *contents, GError **error)
         lines++;
 
     for (; *lines; lines++) {
-        if (*lines[0] == '\0') {
+        gsize line_length;
+        gchar *line = *lines;
+
+        line_length = strlen(line);
+        if (line_length > 0 && line[line_length - 1] == '\r')
+            line[line_length - 1] = '\0';
+
+        if (line[0] == '\0') {
             lines++;
             break;
-        } else if (is_header(*lines)) {
-            if (!parse_header(*lines, &recipient_list, error)) {
+        } else if (is_header(line)) {
+            if (!parse_header(line, &recipient_list, error)) {
                 g_strfreev(first_lines);
                 return FALSE;
             }
-        } else if (g_ascii_isspace(*lines[0])) {
-            append_header_value(*lines);
+        } else if (g_ascii_isspace(line[0])) {
+            append_header_value(line);
         } else {
             g_set_error(error,
                         MILTER_TEST_SERVER_ERROR,
                         MILTER_TEST_SERVER_ERROR_INVALID_HEADER,
                         "invalid header: <%s>",
-                        *lines);
+                        line);
             g_strfreev(first_lines);
             return FALSE;
         }
@@ -1208,6 +1300,7 @@ parse_mail_contents (const gchar *contents, GError **error)
                                   body_string->len - strlen("\r\n")));
     }
     g_string_free(body_string, TRUE);
+    g_ptr_array_add(chunks, NULL);
     body_chunks = (gchar **)g_ptr_array_free(chunks, FALSE);
 
     if (recipient_list) {
@@ -1329,19 +1422,61 @@ static const GOptionEntry option_entries[] =
      N_("Use SPEC for address on connect. "
         "(unix:PATH|inet:PORT[@HOST]|inet6:PORT[@HOST])"),
      "SPEC"},
+    {"connect-macro", 0, 0, G_OPTION_ARG_CALLBACK, parse_connect_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_connect(). "
+        "To add N macros, use --connect-macro option N times."),
+     "NAME:VALUE"},
     {"helo-fqdn", 0, 0, G_OPTION_ARG_STRING, &helo_host,
      N_("Use FQDN for HELO/EHLO command"), "FQDN"},
-    {"from", 'f', 0, G_OPTION_ARG_STRING, &envelope_from,
-     N_("Use a sender address"), "FROM"},
-    {"recipient", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &recipients,
-     N_("Add a recipient. To add n recipients, use --recipient option n times."),
+    {"helo-macro", 0, 0, G_OPTION_ARG_CALLBACK, parse_helo_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_helo(). "
+        "To add N macros, use --helo-macro option N times."),
+     "NAME:VALUE"},
+    {"from", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &envelope_from,
+     N_("Use FROM as a sender address. "
+         "(Deprecated. Use --envelope-from instead.)"), "FROM"},
+    {"envelope-from", 'f', 0, G_OPTION_ARG_STRING, &envelope_from,
+     N_("Use FROM as a sender address"), "FROM"},
+    {"envelope-from-macro", 0, 0, G_OPTION_ARG_CALLBACK,
+     parse_envelope_from_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_envfrom(). "
+        "To add N macros, use --envelope-from-macro option N times."),
+     "NAME:VALUE"},
+    {"recipient", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING_ARRAY,
+     &recipients,
+     N_("Add a recipient RECIPIENT. "
+        "To add N recipients, use --recipient option N times. "
+        "(Deprecated. Use --envelope-recipient instead."),
      "RECIPIENT"},
+    {"envelope-recipient", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &recipients,
+     N_("Add a recipient RECIPIENT. "
+        "To add N recipients, use --envelope-recipient option N times."),
+     "RECIPIENT"},
+    {"envelope-recipient-macro", 0, 0, G_OPTION_ARG_CALLBACK,
+     parse_envelope_recipient_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_envrcpt(). "
+        "To add N macros, use --envelope-recipient-macro option N times."),
+     "NAME:VALUE"},
+    {"data-macro", 0, 0, G_OPTION_ARG_CALLBACK, parse_data_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_data(). "
+        "To add N macros, use --data-macro option N times."),
+     "NAME:VALUE"},
     {"header", 'h', 0, G_OPTION_ARG_CALLBACK, parse_header_arg,
      N_("Add a header. To add n headers, use --header option n times."),
      "NAME:VALUE"},
+    {"end-of-header-macro", 0, 0, G_OPTION_ARG_CALLBACK,
+     parse_end_of_header_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_eoh(). "
+        "To add N macros, use --end-of-header-macro option N times."),
+     "NAME:VALUE"},
     {"body", 'b', 0, G_OPTION_ARG_STRING_ARRAY, &body_chunks,
      N_("Add a body chunk. To add n body chunks, use --body option n times."),
      "CHUNK"},
+    {"end-of-message-macro", 0, 0, G_OPTION_ARG_CALLBACK,
+     parse_end_of_message_macro_arg,
+     N_("Add a macro that has NAME name and VALUE value on xxfi_eom(). "
+        "To add N macros, use --end-of-message-macro option N times."),
+     "NAME:VALUE"},
     {"unknown", 0, 0, G_OPTION_ARG_STRING, &unknown_command,
      N_("Use COMMAND for unknown SMTP command."), "COMMAND"},
     {"authenticated-name", 0, 0, G_OPTION_ARG_STRING, &authenticated_name,
@@ -1356,8 +1491,7 @@ static const GOptionEntry option_entries[] =
         "(This is used for {auth_author} macro value.)"), "AUTHOR"},
     {"mail-file", 'm', 0, G_OPTION_ARG_CALLBACK, parse_mail_file_arg,
      N_("Use mail placed at PATH as mail content."), "PATH"},
-    {"output-message", 0,
-     G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &output_message,
+    {"output-message", 0, 0, G_OPTION_ARG_NONE, &output_message,
      N_("Output modified message"), NULL},
     {"color", 0, G_OPTION_FLAG_OPTIONAL_ARG,
      G_OPTION_ARG_CALLBACK, parse_color_arg,
@@ -1372,7 +1506,7 @@ static const GOptionEntry option_entries[] =
      N_("Timeout after SECONDS seconds on end-of-message command."), "SECONDS"},
     {"threads", 't', 0, G_OPTION_ARG_INT, &n_threads,
      N_("Create N threads."), "N"},
-    {"verbose", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &verbose,
+    {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
      N_("Be verbose"), NULL},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
      N_("Show version"), NULL},
@@ -1476,6 +1610,29 @@ free_process_data (ProcessData *data)
         g_error_free(data->error);
 }
 
+static void
+add_macro_values (GHashTable *macros, const gchar *name, ...)
+{
+    va_list args;
+
+    va_start(args, name);
+    while (name) {
+        const gchar *value;
+
+        value = va_arg(args, const gchar *);
+        g_hash_table_insert(macros, g_strdup(name), g_strdup(value));
+
+        name = va_arg(args, const gchar *);
+    }
+    va_end(args);
+}
+
+static GHashTable *
+macros_new (void)
+{
+    return g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+}
+
 static gboolean
 pre_option_parse (GOptionContext *option_context,
                   GOptionGroup *option_group,
@@ -1485,9 +1642,103 @@ pre_option_parse (GOptionContext *option_context,
     option_headers = milter_headers_new();
     use_color = milter_utils_guess_console_color_usability();
 
+    connect_macros = macros_new();
+    helo_macros = macros_new();
+    envelope_from_macros = macros_new();
+    envelope_recipient_macros = macros_new();
+    data_macros = macros_new();
+    end_of_header_macros = macros_new();
+    end_of_message_macros = macros_new();
+
     return TRUE;
 }
 
+static void
+apply_options_to_connect_macros (void)
+{
+    add_macro_values(connect_macros,
+                     "{daemon_name}", g_get_prgname(),
+                     "{if_name}", "localhost",
+                     "{if_addr}", "127.0.0.1",
+                     "j", "mail.example.com",
+                     NULL);
+}
+
+static void
+apply_options_to_helo_macros (void)
+{
+    add_macro_values(helo_macros,
+                     "{tls_version}", "0",
+                     "{cipher}", "0",
+                     "{cipher_bits}", "0",
+                     "{cert_subject}", "cert_subject",
+                     "{cert_issuer}", "cert_issuer",
+                     NULL);
+}
+
+static void
+apply_options_to_envelope_from_macros (void)
+{
+    add_macro_values(envelope_from_macros,
+                     "i", "i",
+                     "{mail_mailer}", "mail_mailer",
+                     "{mail_host}", "mail_host",
+                     "{mail_addr}", "mail_addr",
+                     NULL);
+
+    if (authenticated_name)
+        add_macro_values(envelope_from_macros,
+                         "{auth_authen}", authenticated_name,
+                         NULL);
+    if (authenticated_type)
+        add_macro_values(envelope_from_macros,
+                         "{auth_type}", authenticated_type,
+                         NULL);
+    if (authenticated_author)
+        add_macro_values(envelope_from_macros,
+                         "{auth_author}", authenticated_author,
+                         NULL);
+}
+
+static void
+apply_options_to_envelope_recipient_macros (void)
+{
+    add_macro_values(envelope_recipient_macros,
+                     "{rcpt_mailer}", "rcpt_mailer",
+                     "{rcpt_host}", "rcpt_host",
+                     NULL);
+}
+
+static void
+apply_options_to_data_macros (void)
+{
+}
+
+static void
+apply_options_to_end_of_header_macros (void)
+{
+}
+
+static void
+apply_options_to_end_of_message_macros (void)
+{
+    add_macro_values(end_of_message_macros,
+                     "i", "message-id",
+                     NULL);
+}
+
+static void
+apply_options_to_macros (void)
+{
+    apply_options_to_connect_macros();
+    apply_options_to_helo_macros();
+    apply_options_to_envelope_from_macros();
+    apply_options_to_envelope_recipient_macros();
+    apply_options_to_data_macros();
+    apply_options_to_end_of_header_macros();
+    apply_options_to_end_of_message_macros();
+}
+
 static gboolean
 post_option_parse (GOptionContext *option_context,
                    GOptionGroup *option_group,
@@ -1530,6 +1781,8 @@ post_option_parse (GOptionContext *option_context,
         body_chunks = g_strsplit(body, ",", -1);
     }
 
+    apply_options_to_macros();
+
     return TRUE;
 }
 
@@ -1538,6 +1791,21 @@ free_option_values (void)
 {
     if (option_headers)
         g_object_unref(option_headers);
+
+    if (connect_macros)
+        g_hash_table_unref(connect_macros);
+    if (helo_macros)
+        g_hash_table_unref(helo_macros);
+    if (envelope_from_macros)
+        g_hash_table_unref(envelope_from_macros);
+    if (envelope_recipient_macros)
+        g_hash_table_unref(envelope_recipient_macros);
+    if (data_macros)
+        g_hash_table_unref(data_macros);
+    if (end_of_header_macros)
+        g_hash_table_unref(end_of_header_macros);
+    if (end_of_message_macros)
+        g_hash_table_unref(end_of_message_macros);
 }
 
 static void
@@ -1772,7 +2040,7 @@ print_body (Message *message)
     if (message->replaced_body_string->len > 0) {
         body_string = message->replaced_body_string->str;
         body_size = message->replaced_body_string->len;
-    }else {
+    } else {
         body_string = message->body_string->str;
         body_size = message->body_string->len;
     }
@@ -1790,10 +2058,19 @@ print_body (Message *message)
                                     &bytes_read,
                                     &bytes_written,
                                     &error);
-        if (!translated_body)
-            g_print("%s\n", body_string);
-        else
+        if (error) {
+            g_print("Error:---------------------------------\n");
+            g_print("%s\n", error->message);
+            g_print("---------------------------------------\n");
+            g_error_free(error);
+        }
+
+        if (translated_body) {
             g_print("%s\n", translated_body);
+            g_free(translated_body);
+        } else {
+            g_print("%s\n", body_string);
+        }
 
         if (charset)
             g_free(charset);
@@ -1832,7 +2109,16 @@ print_status (MilterServerContext *context, ProcessData *data)
         g_type_class_unref(enum_class);
     }
 
-    g_print("status: %s\n", status_name);
+    g_print("status: %s", status_name);
+    if (data->reply_code > 0 ||
+        data->reply_extended_code ||
+        data->reply_message) {
+        g_print(" [%d]<%s>(%s)",
+                data->reply_code,
+                MILTER_LOG_NULL_SAFE_STRING(data->reply_extended_code),
+                MILTER_LOG_NULL_SAFE_STRING(data->reply_message));
+    }
+    g_print("\n");
 }
 
 static void
@@ -1920,6 +2206,10 @@ main (int argc, char *argv[])
     GOptionContext *option_context;
     GOptionGroup *main_group;
 
+#ifdef HAVE_LOCALE_H
+    setlocale(LC_ALL, "");
+#endif
+
     milter_init();
     milter_server_init();
 
